#include "DXUT.h"
#include "FullScreenQuad.h"

struct QuadVertex
{
    D3DXVECTOR3 Position;
    D3DXVECTOR2 Texcoord;
};

QuadVertex QuadVertexes[] = 
{
    D3DXVECTOR3( -1.0f,  1.0f,  0.0f ), D3DXVECTOR2( 0.0f, 0.0f ),
    D3DXVECTOR3( -1.0f, -1.0f,  0.0f ), D3DXVECTOR2( 0.0f, 1.0f ),
    D3DXVECTOR3(  1.0f,  1.0f,  0.0f ), D3DXVECTOR2( 1.0f, 0.0f ),
    D3DXVECTOR3(  1.0f, -1.0f,  0.0f ), D3DXVECTOR2( 1.0f, 1.0f ),
};

UINT QuadIndexes[] = 
{
    0, 2, 1,
    1, 2, 3
};

ID3D10InputLayout*                                  g_pQuadInputLayout         = NULL;
ID3D10Buffer*                                       g_pQuadVertexBuffer        = NULL;
ID3D10Buffer*                                       g_pQuadIndexBuffer         = NULL;

UINT g_QuadStride;
UINT g_QuadOffset;

const WCHAR g_EffectFileName[]        = L"Inpainting.fx";
const WCHAR g_OriginalImageFileName[] = L"Portrait.png";
const WCHAR g_MaskFileName[]          = L"PortraitMask.png";

HRESULT InitFullScreenQuad(ID3D10Device *pd3dDevice, D3D10_PASS_DESC passDesc)
{
    HRESULT hr = S_OK;

    // Input Layout Init
    static D3D10_INPUT_ELEMENT_DESC quadDesc[] = 
    {
        { "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0,  0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT,    0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    };

    UINT NumElements = sizeof( quadDesc ) / sizeof( quadDesc[0] );

    V_RETURN( pd3dDevice->CreateInputLayout( quadDesc, NumElements, passDesc.pIAInputSignature, passDesc.IAInputSignatureSize, &g_pQuadInputLayout) );

    pd3dDevice->IASetInputLayout( g_pQuadInputLayout );

    // Vertex Buffer Init
    static D3D10_BUFFER_DESC quadVertexesDesc;

    UINT NumVertexes = sizeof( QuadVertexes ) / sizeof( QuadVertex );

    quadVertexesDesc.ByteWidth       = sizeof( QuadVertex ) * NumVertexes;
    quadVertexesDesc.BindFlags       = D3D10_BIND_VERTEX_BUFFER;
    quadVertexesDesc.CPUAccessFlags  = 0;
    quadVertexesDesc.Usage           = D3D10_USAGE_DEFAULT;
    quadVertexesDesc.MiscFlags       = 0;

    D3D10_SUBRESOURCE_DATA InitData;
    InitData.pSysMem           = QuadVertexes;
    InitData.SysMemPitch       = 0;
    InitData.SysMemSlicePitch  = 0;

    V_RETURN( pd3dDevice->CreateBuffer( &quadVertexesDesc, &InitData, &g_pQuadVertexBuffer ) );

    g_QuadOffset = 0;
    g_QuadStride = sizeof( QuadVertex );

    // Index Buffer Init
    static D3D10_BUFFER_DESC quadIndexesDesc;

    UINT NumIndexes = sizeof( QuadIndexes ) / sizeof( UINT) ;

    quadIndexesDesc.ByteWidth       = sizeof( UINT ) * NumIndexes;
    quadIndexesDesc.BindFlags       = D3D10_BIND_INDEX_BUFFER;
    quadIndexesDesc.CPUAccessFlags  = 0;
    quadIndexesDesc.Usage           = D3D10_USAGE_DEFAULT;
    quadIndexesDesc.MiscFlags       = 0;

    InitData.pSysMem           = QuadIndexes;   
    InitData.SysMemPitch       = 0;
    InitData.SysMemSlicePitch  = 0;

    V_RETURN( pd3dDevice->CreateBuffer( &quadIndexesDesc, &InitData, &g_pQuadIndexBuffer ) );

    return hr;
}

void ReleaseFullScreenQuad(void)
{
    SAFE_RELEASE( g_pQuadVertexBuffer );
    SAFE_RELEASE( g_pQuadIndexBuffer  );
    SAFE_RELEASE( g_pQuadInputLayout  );
}

void DrawFullScreenQuad(ID3D10Device* pd3dDevice)
{
    pd3dDevice->IASetInputLayout( g_pQuadInputLayout );
    
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

    pd3dDevice->IASetIndexBuffer( g_pQuadIndexBuffer, DXGI_FORMAT_R32_UINT, 0);

    pd3dDevice->IASetVertexBuffers(0, 1, &g_pQuadVertexBuffer, &g_QuadStride, &g_QuadOffset);
    
    pd3dDevice->DrawIndexed(6, 0, 0);
}

void DrawFullScreenQuad(ID3D10Device* pd3dDevice, unsigned int Width, unsigned int Height)
{
	D3D10_VIEWPORT vpOld[1];
    UINT nViewPorts = 1;
    pd3dDevice->RSGetViewports( &nViewPorts, vpOld );

    // Setup the viewport to match the backbuffer
    D3D10_VIEWPORT vp;
    vp.Width    = Width;
    vp.Height   = Height;
    vp.MinDepth = 0.0f;
    vp.MaxDepth = 1.0f;
    vp.TopLeftX = 0;
    vp.TopLeftY = 0;
    pd3dDevice->RSSetViewports( 1, &vp );

    DrawFullScreenQuad(pd3dDevice);

	// Restore the old viewport
    pd3dDevice->RSSetViewports( nViewPorts, vpOld );
}