//--------------------------------------------------------------------------------------
// File: HDRSetup.fx
// Author: Alexander Kharlamov
// Email: akharlamov@nvidia.com
//
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

// The per-color weighting to be used for luminance calculations in RGB order.
static const float3 LUMINANCE = float3(0.2125f, 0.7154f, 0.0721f);
static const float  MIDDLE_GRAY = 0.6f;
static const float  LUM_WHITE = 1.5f;
static const float  BRIGHT_THRESHOLD = 0.5f;

Texture2D s0;
Texture2D s1;
Texture2D s2;

SamplerState sPointWrap
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = Wrap;
    AddressV = Wrap;
};

SamplerState sLinearClamp
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

SamplerState sPointClamp
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = Clamp;
    AddressV = Clamp;
};

shared cbuffer cb2
{
	float4 g_avOffsetsHorizontal[15];
	float4 g_avOffsetsVertical[15];
	float4 g_avSampleWeights[15];
};

DepthStencilState RenderDepthNormal
{
    DepthEnable = true;
    DepthWriteMask = ALL;
    DepthFunc = LESS_EQUAL;
};

RasterizerState NoCulling
{
    CullMode = NONE;
};

BlendState NoBlending
{
	BlendEnable[0] = false;
};

DepthStencilState NoDepthStencil
{
    DepthEnable   = false;
    StencilEnable = false;
};

struct PSInQuad
{
	float4 Position : SV_Position;
    float2 TexCoord : TexCoord0;	
};
struct VSInQuad
{
    float4 Position : POSITION;
    float2 TexCoord : TexCoord;
};

PSInQuad VSMainQuad(VSInQuad In) 
{
    PSInQuad Out;
    Out.Position = In.Position;
    Out.TexCoord = In.TexCoord;

    return Out;
}

VertexShader vsCompiledQuad = CompileShader( vs_4_0, VSMainQuad() );

//-----------------------------------------------------------------------------
// Name: BrightPassPS                                    
// Desc: Scale down the source texture from the average of 3x3 blocks
//-----------------------------------------------------------------------------
float4 BrightPassPS( PSInQuad input ) : SV_TARGET
{   
    float3 vColor = 0.0f;
    float4 vLum = s1.Sample( sPointWrap, float2(0, 0) );
    float  fLum;
    
    fLum = vLum.r;
   
	[unroll]
    for( int y = -1; y <= 1; y++ ) 
    {
		[unroll]
        for( int x = -1; x <= 1; x++ )
        {
            // Compute the sum of color values
            float4 vSample = s0.Sample( sLinearClamp, input.TexCoord, int2(x,y) );
            vColor += vSample.rgb;
        }
    }
    
    // Divide the sum to complete the average
    vColor /= 9.0f;
 
    // Bright pass and tone mapping
    vColor = max( 0.0f, vColor - BRIGHT_THRESHOLD );
    vColor *= MIDDLE_GRAY / (fLum + 0.001f);
    vColor *= (1.0f + vColor/LUM_WHITE);
    vColor /= (1.0f + vColor);
    
    return float4(vColor, 1.0f);
}

//-----------------------------------------------------------------------------
// Name: DownScale2x2_Lum                                    
// Desc: Scale down the source texture from the average of 3x3 blocks and
//       convert to grayscale
//-----------------------------------------------------------------------------
float4 DownScale2x2_Lum ( PSInQuad input ) : SV_TARGET
{    
    float3 vColor;
    float fAvg = 0.0f;
    
	[unroll]
    for( int y = -1; y < 1; y++ )
    {
		[unroll]
        for( int x = -1; x < 1; x++ )
        {
            // Compute the sum of color values
            vColor = s0.Sample( sLinearClamp, input.TexCoord, int2(x,y) ).rgb;
            fAvg += dot( vColor.rgb, LUMINANCE );
        }
    }
    
    fAvg *= 0.25f;

    return float4(fAvg, fAvg, fAvg, 1.0f);
}

//-----------------------------------------------------------------------------
// Name: DownScale3x3                                  
// Desc: Scale down the source texture from the average of 3x3 blocks
//-----------------------------------------------------------------------------
float4 DownScale3x3( PSInQuad input ) : SV_TARGET
{
    float fAvg = 0.0f; 
    float vColor;
    
	[unroll]
    for( int y = -1; y <= 1; y++ )
    {
		[unroll]
        for( int x = -1; x <= 1; x++ )
        {
            // Compute the sum of color values
            vColor = s0.Sample( sLinearClamp, input.TexCoord, int2(x,y) ).x;
            fAvg += vColor.r; 
        }
    }
    
    // Divide the sum to complete the average
    fAvg /= 9.0f;

    return float4(fAvg, fAvg, fAvg, 1.0f);
}

//-----------------------------------------------------------------------------
// Name: Bloom                                     
// Desc: Blur the source image along the horizontal using a gaussian
//       distribution
//-----------------------------------------------------------------------------
float4 BloomHorizontal( PSInQuad input ) : SV_TARGET
{    
    float4 vSample = 0.0f;
    float4 vColor  = 0.0f;
    float2 vSamplePosition;
    
	[unroll]
    for( int iSample = 0; iSample < 15; iSample++ )
    {
        // Sample from adjacent points
        vSamplePosition = input.TexCoord + g_avOffsetsHorizontal[iSample].xy;
        vColor = s0.Sample( sLinearClamp, vSamplePosition );
        
        vSample += g_avSampleWeights[iSample] * vColor;
    }
    
    return vSample;
}

float4 BloomVertical( PSInQuad input ) : SV_TARGET
{    
    float4 vSample = 0.0f;
    float4 vColor  = 0.0f;
    float2 vSamplePosition;
    
	[unroll]
    for( int iSample = 0; iSample < 15; iSample++ )
    {
        // Sample from adjacent points
        vSamplePosition = input.TexCoord + g_avOffsetsVertical[iSample].xy;
        vColor = s0.Sample( sLinearClamp, vSamplePosition );
        
        vSample += g_avSampleWeights[iSample] * vColor;
    }
    
    return vSample;
}

//-----------------------------------------------------------------------------
// Name: ToneMappingPS
//-----------------------------------------------------------------------------
float4 ToneMappingPS( PSInQuad input ) : SV_TARGET
{   
    float4 vColor = s0.Sample( sPointClamp, input.TexCoord );
    float  vLum   = s1.Sample( sPointClamp, float2(0,0) ).r;
    float3 vBloom = s2.Sample( sLinearClamp, input.TexCoord );
    
    // Tone mapping
    vColor.rgb *= MIDDLE_GRAY / ( vLum + 0.001f );
    vColor.rgb *= ( 1.0f + vColor / LUM_WHITE );
    vColor.rgb /= ( 1.0f + vColor );
    
    vColor.rgb += 0.6f * vBloom;
	vColor.a = 1.0f;
    
    return vColor;
}

technique10 DownScale
{
    pass DownScale2x2
    {
        SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetDepthStencilState( NoDepthStencil, 0 );
		
		SetVertexShader  ( vsCompiledQuad );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, DownScale2x2_Lum() ) );
    }

    pass DownScale3x3
    {
        SetDepthStencilState( NoDepthStencil, 0 );
		SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		
		SetVertexShader  ( vsCompiledQuad );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, DownScale3x3() ) );
    }
}

technique10 BrightPass
{
    pass p0
    {
        SetDepthStencilState( NoDepthStencil, 0 );
		SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		
		SetVertexShader  ( vsCompiledQuad );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, BrightPassPS() ) );
    }
}

technique10 ToneMapping
{
    pass p0
    {
        SetDepthStencilState( NoDepthStencil, 0 );
		SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
				
		SetVertexShader  ( vsCompiledQuad );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, ToneMappingPS() ) );
    }
}

//-----------------------------------------------------------------------------
// Technique: Bloom
//-----------------------------------------------------------------------------
technique10 Bloom
{
    pass p0
    {
        SetDepthStencilState( NoDepthStencil, 0 );
        SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		
		SetVertexShader  ( vsCompiledQuad );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, BloomHorizontal() ) );
    }

	pass p1
    {
        SetDepthStencilState( NoDepthStencil, 0 );
        SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		
		SetVertexShader  ( vsCompiledQuad );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, BloomVertical() ) );
    }
}

technique10 SetHDRStates
{
    pass p0
    {
		SetRasterizerState( NoCulling );
		SetDepthStencilState( RenderDepthNormal, 0 );
        SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
    }
}