//--------------------------------------------------------------------------------------
// File: ParallaxMapping.fx
// Author: Alexander Kharlamov
// Email: akharlamov@nvidia.com
//
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------
#include "defines.h"

Texture2D       t2dDiffuse;
Texture2D       t2dBump;
Texture2D       t2dHeight;
TextureCube     t2dShadowMap;

float4x4   g_mCubeView[6]     : CubeView;
float4x4   g_mCubeProj        : CubeProj;

float4x4   g_mModel           : Model;                  // Model matrix for object
float4x4   g_mViewProj        : ViewProjection;         // View * Projection matrix for object
float4x4   g_mModelViewProj   : ModelViewProjection;    // Model * View * Projection matrix

float3 g_CameraEye;
float4 g_LightPos = { 0.0f, 0.0f, 0.0f, 1.0f };
float3 g_LightDir = { 1.0f, 1.0f, 1.0f };

float2 g_TexScale = {4.0f, 20.0f};

static const float4 g_matDiffuse   = { 1.00f, 1.00f, 1.00f, 1.0f };
static const float4 g_matAmbient   = { 0.35f, 0.35f, 0.35f, 0.0f };
static const float4 g_matSpecular  = {  0.5f,  0.5f,  0.5f, 1.0f };

static const float  g_ShadowHardness = 0.75f;
static const float  g_Offset         = 0.01f;
static const float  g_ExtrudeScale   = 0.05f;

float  g_Sign            =   1.00f;
float  g_ExtrudeFin      =   0.15f;
float  g_ExtrudeAngle    =   0.01f;
float  g_CoarseSteps     =   8.00f;
float  g_DetailSteps     =   8.00f;
float  g_CoarseStepsMin  =   8.00f;
float  g_DetailStepsMin  =   8.00f;
float  g_CoarseStepsMax  =  24.00f;
float  g_DetailStepsMax  =  24.00f;
float  g_ParallaxClamp	 =   2.00f;

float  g_Boolean1 = 0.0f;
float  g_Boolean2 = 0.0f;
float  g_Boolean3 = 0.0f;

SamplerComparisonState  sClampCmp
{
	ComparisonFunc = LESS_EQUAL;
    Filter = COMPARISON_MIN_MAG_MIP_LINEAR;
};

SamplerState  sCubeMap
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
	AddressW = Clamp;
};

SamplerState  sClamp
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

SamplerState sLinear
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = Mirror;
	AddressV = Mirror;
};

RasterizerState DisableCulling
{
	FillMode = SOLID;
	CullMode = NONE;
};

RasterizerState Wireframe
{
	CullMode = NONE;
	FillMode = WIREFRAME;
};

struct VS_BOX_INPUT
{
    float4 Position  : POSITION; 
	float3 Normal    : NORMAL; 
	float2 TexCoord  : TEXCOORD;
	float3 Tangent   : TANGENT;
};

struct PS_BOX_INPUT
{
    float4 Position  : SV_POSITION;
    float2 TexCoord  : TEXCOORD0;
	float3 PosWS     : TEXCOORD1;
	float3 Normal    : TEXCOORD2;
	float3 Tangent   : TEXCOORD3;
};

struct VS_INPUT
{
    float4 Position  : POSITION;
    float3 Normal    : NORMAL;  
    float2 TexCoord  : TEXCOORD;
    float3 Tangent   : TANGENT;
};

struct GS_INPUT
{
    float4 Position  : POSITION;
    float2 TexCoord  : TEXCOORD0;
    float3 Normal    : NORMAL;
    float3 Tangent   : TANGENT;
};

struct PS_INPUT
{
    float4 Position  : SV_POSITION;
    float2 TexCoord  : TEXCOORD0;
    float3 Normal    : TEXCOORD1;
    float3 Tangent   : TEXCOORD2;
	float4 PosWS     : TEXCOORD3;
};

float2 ParallaxVector(float3 ViewTS)
{
	float  ParallaxLength    = length   (ViewTS.xy) / (length(ViewTS.z) + g_Offset);
    float2 ParallaxDirection = normalize(ViewTS.xy);
    ParallaxDirection = ParallaxDirection * g_ExtrudeScale * min( ParallaxLength, g_ParallaxClamp );
    return ParallaxDirection;
}

float3 ToTangentSpace(float3 Normal, float3 Tangent, float3 Vector)
{
	float3 Bitangent = normalize(cross(Normal, Tangent));
	//float3 Bitangent = normalize(cross(Tangent, Normal));
    
    float3x3 TangentTransform = float3x3(Tangent, Bitangent, Normal);
    
    float3   VectorTS = mul(TangentTransform, Vector);

    return VectorTS;
}

float  Shadowing(float3 PosWS)
{
    float3 LightDirWS;
    float4 PosLS;
	float3 offset;

    int cubeMapNum = 0;

    LightDirWS = normalize(PosWS - g_LightPos.xyz);    

	PosLS.xyz = LightDirWS;

    float maxCoord = max( abs( PosLS.x ), max( abs( PosLS.y ), abs( PosLS.z ) ) );
        
    [flatten]
    if( maxCoord == abs( PosLS.x ) )
    {
        cubeMapNum = PosLS.x > 0 ? 0 : 1;
		offset.x = 0;
		offset.y = 1.0f / CUBE_MAP_SIZE;
		offset.z = 1.0f / CUBE_MAP_SIZE;
    }
    
    [flatten]
    if( maxCoord == abs( PosLS.y ) )
    {
        cubeMapNum = PosLS.y > 0 ? 2 : 3;
		offset.x = 1.0f / CUBE_MAP_SIZE;
		offset.y = 0;
		offset.z = 1.0f / CUBE_MAP_SIZE;
    }
    
    [flatten]
    if( maxCoord == abs( PosLS.z ) )
    {
        cubeMapNum = PosLS.z > 0 ? 4 : 5;
		offset.x = 1.0f / CUBE_MAP_SIZE;
		offset.y = 1.0f / CUBE_MAP_SIZE;
		offset.z = 0;
    }

    // Apply inverse transformations to a position in light space.
    PosLS = float4( PosWS, 1.0f );
    PosLS = mul( PosLS, g_mCubeView[cubeMapNum] );
    PosLS = mul( PosLS, g_mCubeProj );
    PosLS *= (1.0f / abs( PosLS.w ));

    float shadowSpace = 0.0f;

#define SOFT_N 1.0f
	[unroll]
    for( int i = -SOFT_N; i <= SOFT_N; i++ )
    {
		[unroll]
        for( int j = -SOFT_N; j <= SOFT_N; j++ )
        {
			[unroll]
			for (int k = -SOFT_N; k <= SOFT_N; k++ )
			{
				float3 shift = offset * float3(i, j, k);
				shadowSpace += t2dShadowMap.SampleCmp( sClampCmp, LightDirWS + shift, PosLS.z).r;
			}
        }
    }

	shadowSpace *= 1.0f / ((2.0f * SOFT_N + 1.0f) * (2.0f * SOFT_N + 1.0f) * (2.0f * SOFT_N + 1.0f));
	
	return shadowSpace * g_ShadowHardness + (1.0f - g_ShadowHardness);
}

float4 Lighting( float2 texCoord, float3 LightTS, float3 ViewTS, float Shadow, float mipLevel )
{
   float3 NormalTS = normalize( t2dBump.SampleLevel( sLinear, texCoord, mipLevel).rgb * 2 - 1 );

   float4 BaseColor = t2dDiffuse.SampleLevel( sLinear, texCoord, mipLevel);
   
   LightTS = float3( LightTS.x, LightTS.y, LightTS.z );
   
   float4 Diffuse = saturate( dot( NormalTS, LightTS )) * g_matDiffuse;
    
   float4 Specular = 0;

  float3 ReflectionTS = normalize( 2 * dot( ViewTS, NormalTS ) * NormalTS - ViewTS );
  //float3 ReflectionTS = normalize( -reflect(ViewTS, NormalTS) );
       
  float RdotL = saturate( dot( ReflectionTS, LightTS ));
  Specular = saturate( pow( RdotL, 60 )) * g_matSpecular;

   float4 FinalColor = (( g_matAmbient + Diffuse ) * BaseColor + Specular ) * Shadow; 
   
   return FinalColor;  
}   

float CalculateMipLevel(float2 texcoord)
{
    float2 dx = ddx( texcoord );
    float2 dy = ddy( texcoord );
   
    return log2( max( sqrt(dot(dx, dx)), sqrt(dot(dy, dy)) ) );
}

float TraceLinear(Texture2D Height, inout float2 CurrPoint, inout float CurrHeight, float2 tStep, float hStep, float mipLevel, float iSteps, inout float Intersected)
{
	float tHeight;
	for (float i = 0.0f; i < (int)iSteps; i++ )
    {
		// The trace will stop at the point before the actuall intersection with height field happens
	    tHeight = 1.0f - Height.SampleLevel( sLinear, CurrPoint - tStep, mipLevel ).x;

		if( (CurrHeight - hStep > tHeight) && (CurrHeight > 0) )
	    {
		    CurrHeight -= hStep;
		    CurrPoint  -= tStep;
	    }
	    else
	    {
		    Intersected = 1.0f;
		    i = iSteps;
	    }
    }
	return tHeight;
}

float TraceBinary(Texture2D Height, inout float2 CurrPoint, inout float CurrHeight, float2 tStep, float hStep, float mipLevel, float iSteps, float eps)
{
	// the intersection happens somewhere between [CurrPoint, CurrPoint - tStep]
	float2 pPrev = CurrPoint;            // the current point will be considered Previous
	float2 pNext = CurrPoint  - tStep;   // the point were intersection happens will be considered Next
	float  hPrev = CurrHeight;           
	float  hNext = CurrHeight - hStep;

	float tHeight = 0.0f;

	for (float i = 0.0f; i < (int)iSteps; i++ )
	{
		// the intersection happens somewhere between [pPrev, pNext]
		CurrHeight = (hNext + hPrev) * 0.5f; 
		CurrPoint  = (pNext + pPrev) * 0.5f;

		tHeight = 1.0f - Height.SampleLevel( sLinear, CurrPoint, mipLevel ).x;			
		float delta = CurrHeight - tHeight;
		
		if (abs(delta) > eps)
		{	        
			if ( CurrHeight > tHeight )
			{
				pPrev = CurrPoint;
				hPrev = CurrHeight;
			}
			else
			{
				pNext = CurrPoint;
				hNext = CurrHeight;
			}
		}
		else
		{ 
			i = iSteps;
		}
	}   

	CurrPoint  = (pPrev + pNext)  * 0.5f;
	CurrHeight = (hPrev + hNext) * 0.5f;

	return tHeight;
}

float TraceLight(Texture2D Height, float2 currPoint, float2 RayTS, float mipLevel, float iSteps)
{
	float Shadow = Height.SampleLevel( sLinear, currPoint, mipLevel ).x;
	
	float Scale = (iSteps - 1) / iSteps;	
	float Result = (t2dHeight.SampleLevel( sLinear, currPoint + RayTS * Scale, mipLevel ).x - Shadow + Scale ) * 0.58;

	for (float i = 2; i < iSteps; i++)
	{
		Scale = (iSteps - i) / iSteps;
		float Sh = (t2dHeight.SampleLevel( sLinear, currPoint + RayTS * Scale, mipLevel ).x - Shadow + Scale ) *  2 * (i - 1);
		Result = max(Result, Sh);
	}
	Result = Result * g_ShadowHardness + (1.0f - g_ShadowHardness);
	return Result * 0.6f;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Relief Mapping / Parallax Occlusion Mapping / Bump Mapping code ////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
PS_INPUT VSReliefMapping( VS_INPUT In )
{
	PS_INPUT Out = (PS_INPUT)0;
    
    Out.TexCoord   = In.TexCoord.yx * g_TexScale;

    Out.Normal  =  mul(In.Normal,  (float3x3)g_mModel);
    Out.Tangent =  mul(In.Tangent, (float3x3)g_mModel);

    float3 WorldPos = mul(In.Position.xyz, (float3x3)g_mModel); 
	
	Out.PosWS.xyz = WorldPos; 
	
	Out.PosWS.w = 1.0f;

    Out.Position = mul(float4(WorldPos, 1), g_mViewProj);

    return Out;
}

float4 PSReliefMapping( PS_INPUT In ) : SV_Target
{
	float3 ViewWS = g_CameraEye - In.PosWS.xyz;

	float3 Normal  = normalize(In.Normal);
	float3 Tangent = normalize(In.Tangent);

	float3 ViewTS = ToTangentSpace(Normal, Tangent, ViewWS);

	float2 texShift = ParallaxVector(ViewTS);

#ifdef N_DOT_V_LERPING    
	float fCoarseSteps = lerp(g_CoarseStepsMin, g_CoarseStepsMax, abs(dot( Normal, normalize(ViewTS) )) );
	float fDetailSteps = lerp(g_DetailStepsMin, g_DetailStepsMax, abs(dot( Normal, normalize(ViewTS) )) );
#else
    float fCoarseSteps = g_CoarseStepsMax + 1;
    float fDetailSteps = g_DetailStepsMax + 1;
#endif

	float2 currPoint = texShift * g_Sign + In.TexCoord; 
    float2 tStep = texShift / float( fCoarseSteps );

    float hStep = 1.0f / float( fCoarseSteps );
    float currHeight  = 1.0f;
    
    float textureHeight = 1.0f;

    float i;
    float j = 0.0f;

	float mipLevel = CalculateMipLevel( In.TexCoord );

	textureHeight = TraceLinear(t2dHeight, currPoint, currHeight, tStep, hStep, 0, fCoarseSteps, j);

#ifdef BINARY_SEARCH
    if (j)	textureHeight = TraceBinary(t2dHeight, currPoint, currHeight, tStep, hStep, 0, fDetailSteps, 0.001f);
#else
    if (j) {
	    tStep /= (fDetailSteps + 1);
	    hStep /= (fDetailSteps + 1);
		
	    textureHeight = TraceLinear(t2dHeight, currPoint, currHeight, tStep, hStep, 0, fDetailSteps, j);
    }
#endif

	float d2 = currHeight - textureHeight;
	float d1 = currHeight - hStep - (1.0f - t2dHeight.SampleLevel( sLinear, currPoint - tStep, 0 ).x);
	float d  = d2 - d1;

	float parallaxlength = ((currHeight - hStep) * d2 - (currHeight) * d1) / d;

#ifdef SHOW_PARALLAX_AMOUNT	
	return parallaxlength;
#endif

	currPoint = In.TexCoord + texShift * ( parallaxlength - 1.0f + g_Sign );

    float3   LightTS = ToTangentSpace(Normal, Tangent, normalize(g_LightPos.xyz - In.PosWS.xyz));

#ifdef SELF_SHADOWING
    float2 vLightRayTS = LightTS.xy * 0.1;
    float fSelfShadowing = clamp(TraceLight(t2dHeight, currPoint, vLightRayTS, 0, 7), 0, 1);
#else
    float fSelfShadowing = 1.0f;
#endif

	float4 Out;

#ifdef OFFSET_SHADOW
		Out = Lighting( currPoint, normalize(LightTS), ViewTS, fSelfShadowing, mipLevel ) * Shadowing(In.PosWS.xyz + Normal * 0.1f + normalize(ViewWS) * ( parallaxlength - 1.0f + g_Sign ));
#else
		Out = Lighting( currPoint, normalize(LightTS), ViewTS, fSelfShadowing, mipLevel ) * Shadowing(In.PosWS.xyz + Normal * 0.1f );
#endif
    
	return Out;
}

float4 PSBumpMapping( PS_INPUT In ) : SV_Target
{ 
	float mipLevel = CalculateMipLevel( In.TexCoord );
	
	float3 ViewWS = g_CameraEye - In.PosWS.xyz;

	float3 Normal  = normalize(In.Normal);
	float3 Tangent = normalize(In.Tangent);

	float3 ViewTS = ToTangentSpace(Normal, Tangent, ViewWS);

    float3 LightTS = ToTangentSpace(Normal, Tangent, normalize(g_LightPos.xyz - In.PosWS.xyz));

#ifdef SELF_SHADOWING
	float2 vLightRayTS = LightTS.xy * 0.1;
    float fSelfShadowing = clamp(TraceLight(t2dHeight, In.TexCoord, vLightRayTS, mipLevel, 7), 0, 1);
#else
    float fSelfShadowing = 1.0f;
#endif

	float4 Out = Lighting( In.TexCoord, normalize(LightTS), ViewTS, fSelfShadowing, mipLevel ) * Shadowing(In.PosWS.xyz + Normal * 0.1f);
    return Out;
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Silhouette Clipping Section/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
GS_INPUT VSSilhouette( VS_INPUT In )
{
    GS_INPUT Out = (GS_INPUT)0;
    
    Out.Position = mul( In.Position, g_mModel ); 
    Out.TexCoord = In.TexCoord.yx * g_TexScale;   

    Out.Normal   = mul(In.Normal,  (float3x3)g_mModel);
    Out.Tangent  = mul(In.Tangent, (float3x3)g_mModel);
    
    return Out;
}

[maxvertexcount(4)] 
void GSSilhouette( triangle GS_INPUT In[3], inout TriangleStream<PS_INPUT> EdgeStream )
{
	int	    iSign[3] = { 0,0,0 };
	float   NdotV[3] = { 0,0,0 };

	float3  newPosition[2] = { 0,0,0, 0,0,0 };
	float3  newNormal  [2] = { 0,0,0, 0,0,0 };
	float3  newTangent [2] = { 0,0,0, 0,0,0 };
	float2  newTexCoord[2] = { 0,0,   0,0 };

	float3 View = 0;

	int isign = 0;

	[unroll]
	for( int i = 0; i < 3; i++ )
	{
		View = g_CameraEye - In[i].Position.xyz;
		
		NdotV[i] = dot( normalize( In[i].Normal ), normalize( View ) );
        iSign[i] = NdotV[i] > 0;
		NdotV[i] = abs( NdotV[i] );
		isign += iSign[i];
	}

	if( isign == 1 || isign == 2 )
	{
		float3 Float;
		
		Float.x = NdotV[0] / ( NdotV[0] + NdotV[1] );
		Float.y = NdotV[1] / ( NdotV[1] + NdotV[2] );
		Float.z = NdotV[2] / ( NdotV[2] + NdotV[0] );

		int curIndex = 0;

		if( ( iSign[0] ^ iSign[1] ) != 0 )
		{		
			newTexCoord [curIndex] = lerp( In[0].TexCoord,     In[1].TexCoord,     Float.x );
			newTangent  [curIndex] = lerp( In[0].Tangent,      In[1].Tangent,      Float.x );
			newPosition [curIndex] = lerp( In[0].Position.xyz, In[1].Position.xyz, Float.x );
			newNormal [curIndex++] = lerp( In[0].Normal,       In[1].Normal,       Float.x );
		}

		if( ( iSign[1] ^ iSign[2] ) != 0 )
		{
			newTexCoord [curIndex] = lerp( In[1].TexCoord,     In[2].TexCoord,     Float.y );
			newTangent  [curIndex] = lerp( In[1].Tangent,      In[2].Tangent,      Float.y );
			newPosition [curIndex] = lerp( In[1].Position.xyz, In[2].Position.xyz, Float.y );
			newNormal [curIndex++] = lerp( In[1].Normal,       In[2].Normal,       Float.y );
		}

		if( ( iSign[2] ^ iSign[0] ) != 0 )
		{
			newTexCoord [curIndex] = lerp( In[2].TexCoord,     In[0].TexCoord,     Float.z );
			newTangent  [curIndex] = lerp( In[2].Tangent,      In[0].Tangent,      Float.z );
			newPosition [curIndex] = lerp( In[2].Position.xyz, In[0].Position.xyz, Float.z );
			newNormal [curIndex++] = lerp( In[2].Normal,       In[0].Normal,       Float.z );
		}

		if( curIndex != 0 )
		{ 
			float3 PositionWS[4];	

			PositionWS[0] = newPosition[0]; 
			PositionWS[1] = newPosition[0] + normalize(newNormal[0]) * g_ExtrudeFin;
			PositionWS[2] = newPosition[1]; 
			PositionWS[3] = newPosition[1] + normalize(newNormal[1]) * g_ExtrudeFin;

			PS_INPUT Out = (PS_INPUT)0;

			Out.Position  = mul( float4(PositionWS[0], 1.0f), g_mViewProj );
			Out.TexCoord  = newTexCoord[0];
			Out.Normal    = newNormal[0];
			Out.Tangent   = newTangent[0];
			Out.PosWS	  = float4( PositionWS[0], 0);

			EdgeStream.Append( Out );

			Out.Position  = mul( float4(PositionWS[1], 1.0f), g_mViewProj );
			Out.TexCoord  = newTexCoord[0];
			Out.Normal    = newNormal[0];
			Out.Tangent   = newTangent[0];	
			Out.PosWS	  = float4( PositionWS[1], 1);

			EdgeStream.Append( Out );

			Out.Position  = mul( float4(PositionWS[2], 1.0f), g_mViewProj );
			Out.TexCoord  = newTexCoord[1];
			Out.Normal    = newNormal[1];
			Out.Tangent   = newTangent[1];	
			Out.PosWS	  = float4( PositionWS[2], 0);
			
			EdgeStream.Append( Out );

			Out.Position  = mul( float4(PositionWS[3], 1.0f), g_mViewProj );
			Out.TexCoord  = newTexCoord[1];
			Out.Normal    = newNormal[1];
			Out.Tangent   = newTangent[1];		
			Out.PosWS	  = float4( PositionWS[3], 1);

			EdgeStream.Append( Out );

			EdgeStream.RestartStrip();
		}
	}
}

float4 PSSilhouette( PS_INPUT In ) : SV_Target
{
	float3 ViewWS = g_CameraEye - In.PosWS.xyz;

	float3 Normal  = normalize( In.Normal  );
	float3 Tangent = normalize( In.Tangent );

	float3 ViewTS  = ToTangentSpace(Normal, Tangent, ViewWS);

	float2 texShift = ParallaxVector(ViewTS);
	
#ifdef N_DOT_V_LERPING    
	float fCoarseSteps = lerp(g_CoarseStepsMin, g_CoarseStepsMax, abs(dot( normalize(Normal), normalize(ViewWS) )) );
	float fDetailSteps = lerp(g_DetailStepsMin, g_DetailStepsMax, abs(dot( normalize(Normal), normalize(ViewWS) )) );
#else
    float fCoarseSteps = g_CoarseStepsMax;
    float fDetailSteps = g_DetailStepsMax;
#endif

	float2 currPoint = texShift * g_Sign + In.TexCoord;
    float2 tStep = texShift / float( fCoarseSteps );

    float hStep = (1.0f - In.PosWS.w) / float( fCoarseSteps );
    float currHeight  = 1.0f;

    float textureHeight;

    float   i;
    float   j = 0;
	float   k = 0;
	float   m = 0;

	float mipLevel = CalculateMipLevel( In.TexCoord );

	textureHeight = TraceLinear(t2dHeight, currPoint, currHeight, tStep, hStep, mipLevel, fCoarseSteps, j);

	if (!j)
	{
		textureHeight = TraceLinear(t2dHeight, currPoint, currHeight, tStep, -hStep, mipLevel, fCoarseSteps, k);
	
		j+=k;	
	}

#ifdef BINARY_SEARCH
		if (j)
			textureHeight = TraceBinary(t2dHeight, currPoint, currHeight, tStep, hStep, mipLevel, fDetailSteps, 0.001f);
#else
		if (j)
		{
			tStep /= (fDetailSteps + 1);
			hStep /= (fDetailSteps + 1);
		
			textureHeight = TraceLinear(t2dHeight, currPoint, currHeight, tStep, hStep, mipLevel, fCoarseSteps, m);
			
			j+=m;
		}
#endif
    if (j == 0) discard;

    float3   LightTS = ToTangentSpace(Normal, Tangent, normalize(g_LightPos.xyz - In.PosWS.xyz));
	
#ifdef SELF_SHADOWING
    float fSelfShadowing = clamp(TraceLight(t2dHeight, currPoint, LightTS.xy * 0.1f, mipLevel, 7), 0, 1);
#else
    float fSelfShadowing = 1.0f;
#endif

	float4 Out = Lighting( currPoint, normalize(LightTS), ViewTS, fSelfShadowing, mipLevel ) * Shadowing(In.PosWS.xyz + Normal * 0.1f);
    return Out;
}

//--------------------------------------------------------------------------------------
// Box Vertex Shader
//--------------------------------------------------------------------------------------
PS_INPUT VSBox( VS_INPUT In )
{
    PS_INPUT Out = (PS_INPUT)0;
    
	Out.Position = float4(mul( In.Position.xyz, (float3x3)g_mModel ), 1);
	
	Out.Position = mul( Out.Position, g_mViewProj );
    
	Out.TexCoord = In.TexCoord.yx * g_TexScale;

	Out.PosWS.xyz = mul( In.Position.xyz, (float3x3)g_mModel );
	Out.PosWS.w = 1.0f;

	Out.Normal = In.Normal;

	Out.Tangent = In.Tangent;
    
    return Out;
}

float4 PSBox( PS_INPUT In) : SV_Target
{
	//return float4(Shadowing(In.PosWS), 1);
    return t2dDiffuse.Sample( sLinear, In.TexCoord );
}


//-------------------------------------------------------------------------------------- techniques 
technique10 SilhouetteClipping
{
    pass P0
    {    
        SetVertexShader   ( CompileShader( vs_4_0, VSReliefMapping() ) );
		SetGeometryShader ( NULL );
        SetPixelShader    ( CompileShader( ps_4_0, PSReliefMapping() ) );
		SetRasterizerState( DisableCulling );
    }    
	pass P1
    {    
        SetVertexShader  ( CompileShader( vs_4_0, VSSilhouette() ) );
        SetGeometryShader( CompileShader( gs_4_0, GSSilhouette() ) );
        SetPixelShader   ( CompileShader( ps_4_0, PSSilhouette() ) );
        SetRasterizerState( DisableCulling );
    }
}
technique10 ReliefMapping
{
    pass P0
    {    
        SetVertexShader   ( CompileShader( vs_4_0, VSReliefMapping() ) );
		SetGeometryShader ( NULL );
        SetPixelShader    ( CompileShader( ps_4_0, PSReliefMapping() ) );
        SetRasterizerState( DisableCulling );
    }
}
technique10 BumpMapping
{
    pass P0
    {    
        SetVertexShader   ( CompileShader( vs_4_0, VSReliefMapping() ) );
		SetGeometryShader ( NULL );
        SetPixelShader    ( CompileShader( ps_4_0, PSBumpMapping() ) );
        SetRasterizerState( DisableCulling );
    }
}
technique10 RenderBox
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VSBox() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PSReliefMapping() ) );
    }
}
